@@ -1,6 +1,6 @@
-"""Type definitions for paraDime."""
+"""Type definitions for ParaDime."""
 
-from typing import Tuple, Union, Callable
+from typing import Callable, Literal, Tuple, Union
 
 import numpy as np
 import numpy.typing as npt
@@ -19,10 +19,8 @@ TensorLike = Union[
     torch.Tensor,
 ]
 
+TypeKeyTuples = list[tuple[Union[Literal["data"], Literal["rels"]], str]]
+
 BinaryTensorFun = Callable[[torch.Tensor, torch.Tensor], torch.Tensor]
 
 IndexList = Union[list[int], npt.NDArray[np.integer], torch.Tensor]
-
-Data = Union[
-    np.ndarray, torch.Tensor, dict[str, Union[np.ndarray, torch.Tensor]]
-]
